@@ -415,6 +415,13 @@ run_destroy_handler (GSLockPlug *plug,
         ri->destroyed = TRUE;
 }
 
+static void
+run_keymap_handler (GdkKeymap *keymap,
+                    GSLockPlug *plug)
+{
+        kbd_lock_mode_update (plug, get_kbd_lock_mode ());
+}
+
 /* adapted from GTK+ gtkdialog.c */
 int
 gs_lock_plug_run (GSLockPlug *plug)
@@ -425,6 +432,8 @@ gs_lock_plug_run (GSLockPlug *plug)
         gulong unmap_handler;
         gulong destroy_handler;
         gulong delete_handler;
+        gulong keymap_handler;
+        GdkKeymap *keymap;
 
         g_return_val_if_fail (GS_IS_LOCK_PLUG (plug), -1);
 
@@ -439,6 +448,14 @@ gs_lock_plug_run (GSLockPlug *plug)
                 gtk_widget_show (GTK_WIDGET (plug));
         }
 
+        keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (plug)));
+
+        keymap_handler =
+                g_signal_connect (keymap,
+                                  "state-changed",
+                                  G_CALLBACK (run_keymap_handler),
+                                  plug);
+
         response_handler =
                 g_signal_connect (plug,
                                   "response",
@@ -482,6 +499,7 @@ gs_lock_plug_run (GSLockPlug *plug)
                 g_signal_handler_disconnect (plug, unmap_handler);
                 g_signal_handler_disconnect (plug, delete_handler);
                 g_signal_handler_disconnect (plug, destroy_handler);
+                g_signal_handler_disconnect (plug, keymap_handler);
         }
 
         g_object_unref (plug);
@@ -1296,8 +1314,6 @@ entry_key_press (GtkWidget   *widget,
 {
         restart_cancel_timeout (plug);
 
-        kbd_lock_mode_update (plug, get_kbd_lock_mode ());
-
         /* if the input widget is visible and ready for input
          * then just carry on as usual
          */
@@ -1384,17 +1400,17 @@ create_page_one (GSLockPlug *plug)
         align = gtk_alignment_new (0.5, 0.5, 1, 1);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), align, NULL);
 
-        vbox = gtk_vbox_new (FALSE, 12);
+        vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
         gtk_container_add (GTK_CONTAINER (align), vbox);
 
-        hbox = gtk_hbox_new (FALSE, 6);
+        hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
         gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
         plug->priv->auth_face_image = gtk_image_new ();
         gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_face_image, FALSE, FALSE, 0);
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_face_image), 0, 0);
 
-        vbox2 = gtk_vbox_new (FALSE, 6);
+        vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
         gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 0);
         gtk_container_set_border_width (GTK_CONTAINER (vbox2), 10);
 
@@ -1422,7 +1438,7 @@ create_page_one (GSLockPlug *plug)
         gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_message_label,
                             FALSE, FALSE, 0);
         /* Buttons */
-        plug->priv->auth_action_area = gtk_hbutton_box_new ();
+        plug->priv->auth_action_area = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
 
         gtk_button_box_set_layout (GTK_BUTTON_BOX (plug->priv->auth_action_area),
                                    GTK_BUTTONBOX_END);
@@ -1483,7 +1499,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_frame_set_shadow_type (GTK_FRAME (plug->priv->frame), GTK_SHADOW_OUT);
         gtk_container_add (GTK_CONTAINER (plug), plug->priv->frame);
 
-        plug->priv->vbox = gtk_vbox_new (FALSE, 0);
+        plug->priv->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
         gtk_container_add (GTK_CONTAINER (plug->priv->frame), plug->priv->vbox);
 
         plug->priv->auth_prompt_kbd_layout_indicator = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
